package com.ahai.vgk_game_services

import android.app.Activity
import com.google.android.gms.games.achievement.Achievement
import com.google.gson.Gson
import com.ahai.vgk_game_services.model.AchievementInfo
import com.google.android.gms.games.AchievementsClient
import io.flutter.plugin.common.MethodChannel.Result

class AchievementsController(private val client: AchievementsClient, private var activity: Activity?) {

    companion object {
        const val RC_ACHIEVEMENT_UI = 9003
    }

    fun unlockAchievement(achievementName: String, result: Result) {
        client.unlock(achievementName)
        result.success("success")
    }

    fun revealAchievement(achievementName: String, result: Result) {
        client.reveal(achievementName)
        result.success("success")
    }

    fun incrementAchievement(achievementName: String, step: Int, result: Result) {
        client.increment(achievementName, step)
        result.success("success")
    }

    fun setAchievementSteps(achievementName: String, steps: Int, result: Result) {
        client.setSteps(achievementName, steps)
        result.success("success")
    }

    fun showAchievements(result: Result) {
        client.achievementsIntent.addOnSuccessListener { intent ->
            activity?.startActivityForResult(intent, RC_ACHIEVEMENT_UI)
        }
        result.success("success")
    }

    fun setPercentAchievement(achievementName: String, percent: Int, result: Result) {
        client.setStepsImmediate(achievementName, percent)
        result.success("success")
    }

    fun loadAchievementInfo(forceReload: Boolean, result: Result) {
        client.load(forceReload)
            .addOnCompleteListener { task ->
                val achievementData = task.result?.get()
                if (task.isSuccessful && achievementData != null) {
                    val list = ArrayList<AchievementInfo>()
                    for(a in achievementData) {
                        val type = a.getType()
                        val current_steps = when(type == Achievement.TYPE_INCREMENTAL) {
                            true -> a.getCurrentSteps()
                            false -> null
                        }
                        val total_steps = when(type == Achievement.TYPE_INCREMENTAL) {
                            true -> a.getTotalSteps()
                            false -> null
                        }
                        list.add(AchievementInfo(
                            a.getAchievementId(),
                            a.getName(),
                            a.getDescription(),
                            a.getState(),
                            type,
                            current_steps,
                            total_steps,
                            a.getXpValue()
                        ))
                    }
                    result.success(Gson().toJson(list))
                } else {
                    result.success("Error")
                }
            }
    }
}