package com.ahai.vgk_game_services

import android.app.Activity
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import com.google.android.gms.games.Games
import com.google.android.gms.drive.Drive
import com.google.android.gms.games.SnapshotsClient
import com.google.android.gms.games.SnapshotsClient.DataOrConflict
import com.google.android.gms.games.snapshot.Snapshot
import com.google.android.gms.games.snapshot.SnapshotMetadataChange
import com.google.android.gms.tasks.Continuation
import java.io.IOException

class SavedGamesController(private val snapshotsClient: SnapshotsClient) {

    fun saveSnapShot(name: String, description: String, snapshotData: String, result: Result) {
        val conflictResolutionPolicy = SnapshotsClient.RESOLUTION_POLICY_MOST_RECENTLY_MODIFIED
        snapshotsClient.open(name, true, conflictResolutionPolicy).addOnFailureListener {
            result.success("Error")
          }
          .continueWith<Pair<Snapshot, ByteArray>>(
            Continuation<DataOrConflict<Snapshot>, Pair<Snapshot, ByteArray>> { task ->
              val snapshot = task.result
              snapshot?.data?.let {
                  return@Continuation Pair(it, snapshotData.toByteArray())
              }
              null
            })
          .addOnCompleteListener { task ->
            if (task.isSuccessful && task.result != null) {
              val snapshot = task.result!!.first
              val data = task.result!!.second
              snapshot?.let {
                data?.let {
                  snapshot.snapshotContents.writeBytes(data)
                  val metadataChange = SnapshotMetadataChange.Builder()
                    .setDescription(description)
                    .build()
                  val commitTask = snapshotsClient.commitAndClose(snapshot, metadataChange)
                  commitTask.addOnCompleteListener { task ->
                    if (task.isSuccessful) {
                      result.success("success")
                    } else {
                      result.success("Error")
                    }
                  }
                }
              }
            } else {
              result.success("Error")
            }
          }
      }
    
    fun loadSnapShot(name: String, result: Result) {
        val conflictResolutionPolicy = SnapshotsClient.RESOLUTION_POLICY_MOST_RECENTLY_MODIFIED
        snapshotsClient.open(name, true, conflictResolutionPolicy).addOnFailureListener {
            result.success("Error")
          }
          .continueWith<ByteArray>(Continuation { task ->
            val snapshot = task.result
            try {
              snapshot?.data?.let {
                  return@Continuation it.snapshotContents.readFully()
              }
              return@Continuation null
            } catch (e: IOException) {
            }
            null
          })
          .addOnCompleteListener { task ->
            if (task.isSuccessful) {
              task.result?.let {
                result.success(String(it))
              }
            } else {
              result.success("Error")
            }
          }
      }
    
}